Package: hellometry
Title: Biomass Estimation from Allometric Relationships
Version: 1.0.1
Authors@R: c(
    person(given = "Pierre",
           family = "Rogy",
           role = c("aut", "cre"),
           email = "pierre.rogy@gmail.com",
           comment = c(ORCID = "0000-0002-3002-0059")),
    person(given = "Olivier",
           family = "Dézerald",
           role = "ctb",
           comment = c(ORCID = "0000-0002-9987-9865")),
    person(given = "Gustavo Q.",
           family = "Romero",
           role = "ctb",
           comment = c(ORCID = "0000-0003-3736-4759")),
    person(given = "Fabiola",
           family = "Ospina-Bautista",
           role = "ctb",
           comment = c(ORCID = "0000-0003-2498-1459")),
    person(given = "Sarah",
           family = "Abdelazim",
           role = "ctb",
           comment = c(ORCID = "0000-0002-6379-2733")),
    person(given = "Nicholas A. C.",
           family = "Marino",
           role = "ctb",
           comment = c(ORCID = "0000-0002-5702-5466")),
    person(given = "Diane S.",
           family = "Srivastava",
           role = "ctb",
           comment = c(ORCID = "0000-0003-4541-5595")))
Description: Estimates body size and biomass of organisms from
    allometric relationships. It works with any data that follow a few
    column-naming conventions, using the measurements you supply to impute the
    ones that are missing.
License: GPL (>= 3)
URL: https://github.com/pierrerogy/hellometry
BugReports: https://github.com/pierrerogy/hellometry/issues
Encoding: UTF-8
RoxygenNote: 7.3.2
Imports: dplyr, magrittr, performance, progress, purrr, rlang, stats,
        tibble, tidyr, tidyselect, utils
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-08-20 16:21:00 UTC; pagro
Author: Pierre Rogy [aut, cre] (ORCID: <https://orcid.org/0000-0002-3002-0059>),
  Olivier Dézerald [ctb] (ORCID: <https://orcid.org/0000-0002-9987-9865>),
  Gustavo Q. Romero [ctb] (ORCID:
    <https://orcid.org/0000-0003-3736-4759>),
  Fabiola Ospina-Bautista [ctb] (ORCID:
    <https://orcid.org/0000-0003-2498-1459>),
  Sarah Abdelazim [ctb] (ORCID: <https://orcid.org/0000-0002-6379-2733>),
  Nicholas A. C. Marino [ctb] (ORCID:
    <https://orcid.org/0000-0002-5702-5466>),
  Diane S. Srivastava [ctb] (ORCID:
    <https://orcid.org/0000-0003-4541-5595>)
Maintainer: Pierre Rogy <pierre.rogy@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-24 14:50:02 UTC

# Generated by roxygen2: do not edit by hand

export("%>%")
export("%notin%")
export(bromeliad_inverts_measurements)
export(data_checker)
export(dry_wet)
export(full_estimation_table)
export(hellometry)
export(make_measurement_table)
export(make_trait_table)
export(matcher_of_traits)
export(predict_biomass)
export(r_squared)
export(trini_communities)
importFrom(dplyr,group_by)
importFrom(dplyr,n)
importFrom(dplyr,ntile)
importFrom(dplyr,row_number)
importFrom(dplyr,summarise)
importFrom(magrittr,"%>%")
importFrom(purrr,imap)
importFrom(rlang,.data)
importFrom(rlang,.env)
importFrom(rlang,expr)
importFrom(rlang,sym)
importFrom(rlang,syms)
importFrom(stats,df.residual)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,poisson)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(tidyselect,all_of)

# hellometry

Estimates body size and biomass of organisms from allometric
relationships. `hellometry` works with any dataset that follows a few simple
column-naming conventions, using the measurements you supply to impute the ones
that are missing.

To download the package, run `devtools::install_github("pierrerogy/hellometry", build_vignettes = TRUE)`. The package vignette (`vignette("hellometry")`) contains a worked example, and additional details.

#' Get the R2 of a fitted allometric model
#'
#' Wraps `performance::r2()`, which returns the R2 suited to the model it is
#' given, so that any kind of model can be filtered the same way in
#' `full_estimation_table()`
#'
#' @param model A fitted allometric model
#' @return A single R2 value, NA if none could be computed
#' @examples
#' # Any allometric model will do
#' mod <-
#'   lm(log10(biomass_col) ~ log10(size_col),
#'      data = data.frame(size_col = c(1, 2, 3, 4, 5),
#'                        biomass_col = c(0.1, 0.5, 1.2, 2.4, 4.1)))
#'
#' r_squared(mod)
#' @export
r_squared <- function(model) {

  # Get the R2, which performance::r2() returns as a list
  ## Degenerate models can fail here, so catch that and return nothing
  ret <-
    tryCatch(suppressWarnings(performance::r2(model)),
             error = function(e) NULL)

  # If no R2 came back, return NA so that the model gets filtered out
  if (is.null(ret) || length(ret) == 0)
    return(NA)

  # Keep the first value, i.e. the R2 itself
  ret <-
    as.numeric(ret[[1]])

  # Return
  return(ret)

}

## ----include=FALSE------------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message = FALSE)

## ----flowchart, echo=FALSE, out.width="100%", fig.alt="Flowchart of a hellometry() call, from the input data through data_checker(), make_measurement_table(), full_estimation_table() for sizes then models, and predict_biomass(), to the returned list of three tables."----
knitr::include_graphics("hellometry_flowchart.svg")

## -----------------------------------------------------------------------------
# Load library
library(hellometry)

# We need to define a suite of levels to be used for estimation, i.e. columns in the data, 
# from lowest to coarsest resolution
level_vec  <-
  c("species", "genus", "family", "order")

# Read in the reference measurements, the real body sizes and body masses used to build the models
measurements <-
  bromeliad_inverts_measurements() %>%
  ## Rename columns to those expected by hellometry()
  dplyr::rename(size_col = body_size_mm,
                biomass_col = body_mass_mg,
                biomass_type = mass_type) %>%
  ## size_col holds both numbers and categories, so it must be character;
  ## biomass_col must be numeric for the allometric models
  dplyr::mutate(size_col = as.character(size_col),
                biomass_col = as.numeric(biomass_col))

# Have a look at the reference measurements
dplyr::glimpse(measurements)

# Read in the Trinidadian communities, the taxa we want size and biomass estimates for
communities <-
  trini_communities() %>%
  ## Rename the abundance column and add the columns hellometry() needs
  dplyr::rename(abundance = n) %>%
  ## Here we do not have any measurement for these invertebrates, so size is "unknown"
  ## and biomass NA, to be estimated. We use dry biomass because it tends to be more precise
  dplyr::mutate(size_col = "unknown",
                biomass_col = NA,
                biomass_type = "dry")

# Have a look at the communities to estimate
dplyr::glimpse(communities)

# Combine the reference measurements and the target communities into one table
my_invertebrates <-
  communities %>%
  dplyr::bind_rows(measurements)


# Now we can use the package to estimate sizes and biomasses!

# Get the estimates
my_invertebrates_estimated <-
  hellometry(dats = my_invertebrates, ## The data to be used
              level_vec  = level_vec , ## The taxonomic levels to be used
              biomass_type = "dry") ## Type of biomass to be estimated

# The result is a list of three elements
# - `data` - your data with estimated body sizes and biomasses, and column with 
# information on the level at which were performed the estimation. Note that 
# it also includes the measurements supplied so will need to filter out these rows
dplyr::glimpse(my_invertebrates_estimated$data)
# - `size_estimates` - a tibble with all unique size estimates that were joined to your data
dplyr::glimpse(my_invertebrates_estimated$size_estimates)
# - `model_estimates` - a tibble with all unique allometric models that were joined to your data
dplyr::glimpse(my_invertebrates_estimated$model_estimates)


# But now let's say you do not really want to do estimation on the data, 
# but just see the results of all possible estimations. Fear not, 
# it is easy!
## First compile the measurement table
measurement_table <-
  make_measurement_table(dats = my_invertebrates, ## Data to be used
                         level_vec  = level_vec) ## Taxonomic levels to be used
## Get all possible size estimates
size_estimation_table <-
  full_estimation_table(level_vec  = level_vec ,
                        measurement_table = measurement_table,
                        what = "size_col")
### Have a glimpse of it
dplyr::glimpse(size_estimation_table)

## Get all possible models
model_estimation_table <-
  full_estimation_table(level_vec  = level_vec ,
                        measurement_table = dry_wet(measurement_table,
                                                        biomass_type = "dry"), ## Filter for dry biomass
                        what = "biomass_col")
### Have a glimpse of it
dplyr::glimpse(model_estimation_table)


## -----------------------------------------------------------------------------

# Read in data with traits
trait_data <-
  trini_communities(traits = TRUE)

# The fuzzy traits are every column after the taxonomy, i.e. from the first trait
# column (AS1) up to the last column of the table
trait_columns <-
  trait_data %>%
  ## Get the block of trait columns ..
  dplyr::select(AS1:dplyr::last_col()) %>%
  ## Keep just their names
  names()

# Rebuild a measurement table, this time asking it to keep every taxonomic level
# we want to match on (make_measurement_table() only keeps the levels we give it)
measurement_table <-
  make_measurement_table(dats = my_invertebrates,
                         level_vec = c("species", "genus", "subfamily",
                                       "family", "order", "class"))

# Here we want to match traits from different morphospecies, look for matches 
# from genus up to class
match_levels <-
  c("genus", "subfamily", "family", "order", "class")

# Build a lookup holding one representative trait profile per taxon, at 
# every level combined into a single long table that remembers the level 
# each profile came from
trait_lookup <-
  match_levels %>%
  purrr::map_dfr(.,
                 ~ trait_data %>%
                     ## Drop the taxa that are unnamed at this level
                     dplyr::filter(!is.na(.data[[.x]]), .data[[.x]] != "") %>%
                     ## One row per taxon name, carrying its trait profile
                     dplyr::distinct(key = .data[[.x]],
                                     dplyr::across(dplyr::all_of(trait_columns))) %>%
                     ## Keep a single profile should a taxon appear more than once
                     dplyr::distinct(key, .keep_all = TRUE) %>%
                     ## Tag which taxonomic level this profile belongs to
                     dplyr::mutate(match_level = .x))

# Now hand each measured taxon the trait profile of its most similar relative, i.e.
# the profile found at the finest taxonomic level at which the taxon matches
trait_assignment <-
  measurement_table %>%
  ## Tag each measurement row so we can return its assigned traits later
  dplyr::mutate(.row = dplyr::row_number()) %>%
  ## Keep only that tag and the taxonomy columns we match on
  dplyr::select(.row, dplyr::all_of(match_levels)) %>%
  ## Stretch to one row per measured taxon per taxonomic level
  tidyr::pivot_longer(dplyr::all_of(match_levels),
                      names_to = "match_level", values_to = "key") %>%
  ## Drop the levels where the taxon has no name
  dplyr::filter(!is.na(key), key != "") %>%
  ## Attach every trait profile that matches, at whatever level
  dplyr::inner_join(trait_lookup, 
                    by = c("match_level", "key")) %>%
  ## Rank the levels from finest to coarsest, following match_levels
  dplyr::mutate(.priority = match(match_level, 
                                  match_levels)) %>%
  ## For each measurement row, keep only the finest matching level
  dplyr::group_by(.row) %>%
  dplyr::slice_min(.priority, 
                   n = 1, 
                   with_ties = FALSE) %>%
  dplyr::ungroup() %>%
  ## Return just the row tag and its assigned trait profile
  dplyr::select(.row, dplyr::all_of(trait_columns))

# Finally, join the assigned traits back onto the measurements, and keep only taxa
# that both received traits and are identified to family (our id_col below)
dats <-
  measurement_table %>%
  ## Recreate the same row tag to join on
  dplyr::mutate(.row = dplyr::row_number()) %>%
  ## Bring in each taxon's assigned trait profile
  dplyr::left_join(trait_assignment, by = ".row") %>%
  ## Drop the helper tag
  dplyr::select(-.row) %>%
  ## Keep taxa that got traits and have a family to identify them by
  dplyr::filter(!is.na(.data[[trait_columns[1]]]),
                !is.na(family), family != "")

## Get long format data of which taxa have similar traits
matched_traits <-
  matcher_of_traits(measurement_table = dats,
                    trait_columns = trait_columns,
                    id_col = "family") ## here taxa are identified by family
### See the output
head(matched_traits)

## Get clean list for full_estimation_table, includes matcher_of_traits()
## Here you can see which taxa were grouped together
clean_trait_list <-
  make_trait_table(measurement_table = dats,
                   trait_columns = trait_columns,
                   id_col = "family")
### See the output
clean_trait_list[[1]]

# The last two functions are called in full_estimation_table if you add the trait arguments
## Size estimation table
size_estimation_table_traits <-
  full_estimation_table(level_vec  = c(),
                        measurement_table = dats,
                        traits = TRUE, ## switch for traits
                        trait_columns = trait_columns, ## vector of fuzzy trait columns
                        id_col = "family", ## where to group
                        what = "size_col")
### See the output
head(size_estimation_table_traits)

## Model table
model_estimation_table_traits <-
  full_estimation_table(level_vec  = c(),
                        measurement_table = dry_wet(dats,
                                                    biomass_type = "dry"), ## Filter for dry biomass
                        traits = TRUE, ## traits = TRUE
                        trait_columns = trait_columns, ## vector of fuzzy trait columns
                        id_col = "family", ## column identifying taxa
                        what = "biomass_col")
### See the output
head(model_estimation_table_traits)

#' @keywords internal
"_PACKAGE"

## Namespace imports for functions called without a `pkg::` prefix.
## (Qualified calls such as `dplyr::filter()` only need the package in
## DESCRIPTION Imports, not an importFrom here.)
#' @importFrom dplyr group_by n ntile row_number summarise
#' @importFrom purrr imap
#' @importFrom rlang .data .env expr sym syms
#' @importFrom stats df.residual glm lm poisson predict qnorm qt
#' @importFrom tidyselect all_of
NULL

# Column names used in non-standard evaluation (data-masked dplyr verbs).
# Declared here so R CMD check does not flag them as undefined globals.
utils::globalVariables(c(
  ".", "abundance", "biomass", "biomass_col", "biomass_lower",
  "biomass_upper", "level", "model", "model_level", "model_taxon_name",
  "name", "size_category", "size_col",
  "size_col.x", "size_col.y", "size_level", "size_taxon_name", "stage"
))

#' Set of checks to perform on the data to make sure it can be handled by `hellometry()`
#'
#' Will fail if any condition not met
#'
#' @param dats Dataframe to be used for estimation
#' @param biomass_type "dry"/"wet". Whether the biomass used in inference should
#' be dry (default) or wet. See `dry_wet()` for details.
#' @param model "lm"/"poisson". What kind of allometric model should be computed.
#' See `full_estimation_table()` for details.
#' @return An error if any check fails.
#' @examples
#' # Data named the way the package expects
#' dats <-
#'   data.frame(species = c("sp_a", "sp_b"),
#'              stage = "larva",
#'              abundance = c(2, 5),
#'              size_col = c("3.1", "unknown"),
#'              biomass_col = c(0.4, NA),
#'              biomass_type = "dry")
#'
#' # Passes quietly
#' data_checker(dats, biomass_type = "dry", model = "lm")
#'
#' # Fails on a column that is not named properly
#' try(data_checker(dats["species"], biomass_type = "dry", model = "lm"))
#' @export
data_checker <- function(dats, biomass_type, model){

  # Important columns have proper names
  if("abundance" %notin% colnames(dats))
    stop("Please call column with abundance values 'abundance'")
  if("size_col" %notin% colnames(dats))
    stop("Please call column with specimen measurement values 'size_col'")
  if("stage" %notin% colnames(dats))
    stop("Please call column with life stage (larva/pupa/adult) 'stage'")
  if("biomass_type" %notin% colnames(dats))
    stop("Please call column with biomass type (dry/wet) 'biomass_type'")

  # Biomass kind needs to be dry or wet
  if(biomass_type %notin% c("dry", "wet"))
    stop("Biomass kind has to be 'dry' or 'wet'")

  # Model kind needs to be one we can compute
  if(model %notin% c("lm", "poisson"))
    stop("Model kind has to be 'lm' or 'poisson'")

  # Check for NA values in abundance before processing
  if (any(is.na(dats$abundance))) {
    stop("Abundance contains NA values") }

}

#' Finds which species have matching traits
#'
#' Evaluates all unique trait groupings and returns species that have the same
#' traits +/- 1
#'
#' @param measurement_table A table with the numerical measurements and biomass
#' used to compute allometric lms
#' @param trait_columns List of traits to match, should be column names in measurement_table
#' @param id_col Name of the column holding a unique identifier per species/taxon.
#' Default "species".


#' @return Tibble of four columns: level = "traits", name = focus species,
#' stage of focus species, id = matched species
#' @examples
#' # Fuzzy traits of three species, sp_a and sp_b within +/- 1 of each other
#' measurement_table <-
#'   data.frame(species = rep(c("sp_a", "sp_b", "sp_c"), each = 4),
#'              stage = "larva",
#'              size_col = c(1, 2, 3, 4, 1.2, 2.1, 3.3, 4.2, 8, 9, 10, 11),
#'              biomass_col = c(0.1, 0.4, 1, 2, 0.2, 0.5, 1.1, 2.2, 12, 15, 20, 26),
#'              AS1 = rep(c(1, 2, 5), each = 4),
#'              AS2 = rep(c(3, 3, 9), each = 4))
#'
#' # sp_a and sp_b match each other, sp_c only itself
#' matcher_of_traits(measurement_table, trait_columns = c("AS1", "AS2"))
#' @export
#'
  matcher_of_traits <- function(measurement_table, trait_columns, id_col = "species") {

    # First filter dataset with unique groupings of traits
    measurement_table <-
      measurement_table %>%
      dplyr::select(dplyr::all_of(c(id_col, trait_columns, "stage"))) %>%
      unique()

    # Identifiers of the trait groupings, in the same row order as the matrix
    ids <- measurement_table[[id_col]]

    # Print a little message saying that we are grouping species by traits
    message("Grouping species by trait similarities..")

    # Build a numeric trait matrix once (rows = trait groupings, cols = traits)
    ## Suppress warnings in case of non-numeric coercion, as in the original
    suppressWarnings(
      trait_mat <-
        vapply(measurement_table[trait_columns],
               as.numeric,
               numeric(nrow(measurement_table))))

    # Set progress bar with number of trait groupings
    pb <-
      progress::progress_bar$new(format = "[:bar] :current/:total (:percent)",
                                 total = nrow(measurement_table))
    ## Initiate it
    pb$tick(0)

    # For each row (unique trait grouping) of the filtered dataframe
    ret <-
      purrr::map_dfr(seq_len(nrow(measurement_table)), function(i) {

      ## Traits of the focus species for this row
      focus_traits <- trait_mat[i, ]

      ## Vectorised trait comparison against every grouping at once:
      ## a row matches if every trait differs from the focus by <= 1.
      ## NA differences (missing traits) never match, as before.
      diffs <- abs(sweep(trait_mat, 2, focus_traits, "-"))
      similar <- rowSums(diffs > 1 | is.na(diffs)) == 0

      ## Add tick to progress bar
      pb$tick()

      ## Make a little catch in case we do not find species with similar traits
      if (!any(similar)) {
        ## Returning an empty tibble
        return(tibble::tibble(level = character(),
                              name = character(),
                              stage = character(),
                              id = character()))
      }

      ## But if we get some species with similar traits, return them long format
      tibble::tibble(level = "traits",
                     name = ids[i],
                     stage = measurement_table$stage[i],
                     id = ids[similar])
      })


    # Return
    return(ret)

}

#' Build measurement table to be used for biomass estimations
#'
#' Compiles the table of numerical measurements used to estimate sizes and fit
#' allometric models. The table is built entirely from the data you supply:
#' every row with a numerical `size_col` is kept (expanded by `abundance` so
#' each row represents one individual), carrying its taxonomy along.
#'
#' @param dats Dataframe to be used for estimation.
#' @param level_vec Vector of taxonomic levels to be used in the measurement table.
#' @return A table with measurements and taxonomy, ready to be used for
#'  size and biomass estimation.
#' @examples
#' # Reference measurements for one family, renamed to the columns the package expects
#' measurements <-
#'   bromeliad_inverts_measurements() %>%
#'   dplyr::filter(family == "Culicidae") %>%
#'   dplyr::rename(size_col = body_size_mm,
#'                 biomass_col = body_mass_mg,
#'                 biomass_type = mass_type) %>%
#'   dplyr::mutate(size_col = as.character(size_col),
#'                 biomass_col = as.numeric(biomass_col))
#'
#' # Taxa to estimate, with no measurement of their own
#' communities <-
#'   trini_communities() %>%
#'   dplyr::filter(family == "Culicidae") %>%
#'   dplyr::rename(abundance = n) %>%
#'   dplyr::mutate(size_col = "unknown",
#'                 biomass_col = NA,
#'                 biomass_type = "dry")
#'
#' # The two together are what the package works on
#' dats <-
#'   dplyr::bind_rows(communities, measurements)
#' level_vec <-
#'   c("species", "genus", "family")
#'
#' # One row per measured individual, taxonomy carried along
#' make_measurement_table(dats = dats, level_vec = level_vec)
#' @export
make_measurement_table <- function(dats, level_vec){

  # Keep only rows with a numerical size measurement.
  ## as.numeric() warns on non-numeric ("unknown", "small", ...) values, which
  ## are exactly the rows we drop, so silence the warning.
  suppressWarnings(
    ret <-
      dats %>%
      dplyr::mutate(size_col = as.numeric(size_col)) %>%
      dplyr::filter(!is.na(size_col)) %>%
      ## We should not expect every taxonomic level to be present, so use any_of
      dplyr::select(tidyselect::any_of(c(level_vec,
                                         "stage", "abundance", "size_col",
                                         "biomass_col", "biomass_type"))))

  # Have each row represent an individual
  ret <-
    ret %>%
    ## First make sure all 0s/NAs are 1s so that no row is removed
    dplyr::mutate(abundance = ifelse(abundance == 0 | is.na(abundance),
                                     1, abundance)) %>%
    ## Then use tidyr::uncount() to make each row a unique measurement
    tidyr::uncount(abundance)

  # Return measurement table
  return(ret)

}

#' Estimate biomass from a fitted allometric model
#'
#' Wraps `predict()` for every kind of model `full_estimation_table()` can fit,
#' and returns the biomass of a whole row on the scale of the supplied data
#'
#' Both kinds return a confidence interval, i.e. the uncertainty around the
#' mean predicted biomass. The estimate is scaled to the abundance of the row
#' while still on the scale the model was fitted on (log10 for an lm, the log
#' link for a Poisson glm), and then back-transformed.
#'
#' @param model A fitted allometric model
#' @param size The size to estimate biomass for
#' @param abundance The number of individuals the row holds
#' @return Three values: the biomass estimate, its lower and its upper bound
#' @examples
#' # Any allometric model will do
#' mod <-
#'   lm(log10(biomass_col) ~ log10(size_col),
#'      data = data.frame(size_col = c(1, 2, 3, 4, 5),
#'                        biomass_col = c(0.1, 0.5, 1.2, 2.4, 4.1)))
#'
#' # Biomass of a row holding ten individuals of size 3, with its bounds
#' predict_biomass(mod, size = 3, abundance = 10)
#' @export
predict_biomass <- function(model, size, abundance) {

  # A row with no individuals holds no biomass, and nothing to be uncertain about
  if (abundance == 0)
    return(c(0, 0, 0))

  # Data to estimate biomass for, named the way the models expect
  newdats <-
    data.frame(size_col = size)

  # If we have a Poisson glm
  ## Careful, glms also inherit the "lm" class, so they need to come first
  if (inherits(model, "glm")) {

    ## Get the estimate and its standard error, on the scale of the log link
    ret <-
      predict(model,
              newdata = newdats,
              type = "link",
              se.fit = TRUE)

    ## The log link is undone with exp(), i.e. a base of e
    base <-
      exp(1)

    ## The interval of a glm is a Wald one, so its bounds sit 1.96 SE away
    critical_value <-
      qnorm(0.975)

  } else {

    ## Get the estimate and its standard error, on the log10 scale
    ret <-
      predict(model,
              newdata = newdats,
              se.fit = TRUE)

    ## lms are fitted on log10(biomass_col), so they are undone with a base of 10
    base <-
      10

    ## Same critical value predict() uses for the confidence interval of an lm
    critical_value <-
      qt(0.975,
         df = df.residual(model))
  }

  # Scale the estimate to the abundance of the row, on the fitted scale
  ## Multiplying biomass by abundance means adding to it on the fitted scale
  ## The standard error needs no scaling, as every individual of the row is
  ## estimated from the same model, so they all share the same uncertainty
  fit <-
    ret$fit + log(abundance, base = base)

  # Back-transform the estimate and its bounds
  ## as.numeric() because predict() gives its output the row name of newdats
  ret <-
    as.numeric(base^c(fit,
                      fit - critical_value * ret$se.fit,
                      fit + critical_value * ret$se.fit))

  # Return
  return(ret)

}

#' Wrapper function to estimate size (mm) and biomass (mg) from an input dataset
#'
#' Wrapper function, input your data and get size and measurement. 
#' Please name column with number of specimen "abundance", column with measurement 
#' "size_col", column with life stage (e.g. larva/adult) "stage", column 
#' with biomass type (dry/wet) "biomass_type", and the column with biomass
#' "biomass_col".
#' If you do not have a numerical measurement for a given specimen, the function
#' can do size estimations if you input "small", "medium", "large" or "unknown".
#' In this case, the algorithm will use existing size measurements for the species,
#' and use the size distribution to estimate "small", "medium" and "large" inputs,
#' or a weighted average of all measurements if the input is "unknown". For biomass
#' measurements, just leave NA for those cells you want estimates in.
#'
#' Please make sure that the level_vec argument has levels in increasing order
#' of coarseness, e.g. from species to order.
#'
#'
#' @param dats The input data table, please include columns "abundance",
#' "size_col", "biomass_col", "stage" (larva/pupa/adult).
#' @param level_vec A character vector indicating the taxonomic levels present
#' in your data, in increasing order of coarseness.
#' @param biomass_type "dry"/"wet". Should data used in inference be "dry" for
#' just dry biomass (default), or "wet" for just wet biomass. See `dry_wet()`
#' for more information on how this works.
#' @param model What kind of allometric model should be computed, "lm" (default)
#' for a linear model on the log10-log10 scale, or "poisson" for a Poisson glm
#' with a log link.
#' @param r_square_cutoff_upper Upper cutoff for R2 in allometric models, models with values above it will not be used in estimation. Default is 0.95 to avoid overfit models
#' @param r_square_cutoff_lower Lower cutoff for R2 in allometric models, models with values below it will not be used in estimation. Default is 0.
#' @param p_val_cutoff Upper cutoff for p-value of allometric models, models with p_value above it will not be used in estimation. Default is 0.05.
#' @return A list with three tibbles:
#' - data: the input data with added size and biomass estimates, and new columns 
#'         with the taxonomic level and name of the taxon at which the estimate 
#'         was made.
#' - size_estimates: tibble with size estimates that were used, 
#'                     with columns for taxonomic level, name and size category
#'                     of the estimation.
#' - model_estimates: tibble with biomass models that were used, with columns
#'                      for taxonomic level and name.
#' See `full_estimation_table()` to get all possible size estimates and models
#' for your data
#' @examples
#' # Reference measurements for one family, renamed to the columns the package expects
#' measurements <-
#'   bromeliad_inverts_measurements() %>%
#'   dplyr::filter(family == "Culicidae") %>%
#'   dplyr::rename(size_col = body_size_mm,
#'                 biomass_col = body_mass_mg,
#'                 biomass_type = mass_type) %>%
#'   dplyr::mutate(size_col = as.character(size_col),
#'                 biomass_col = as.numeric(biomass_col))
#'
#' # Taxa to estimate, with no measurement of their own
#' communities <-
#'   trini_communities() %>%
#'   dplyr::filter(family == "Culicidae") %>%
#'   dplyr::rename(abundance = n) %>%
#'   dplyr::mutate(size_col = "unknown",
#'                 biomass_col = NA,
#'                 biomass_type = "dry")
#'
#' # The two together are what the package works on
#' dats <-
#'   dplyr::bind_rows(communities, measurements)
#' level_vec <-
#'   c("species", "genus", "family")
#'
#' res <-
#'   hellometry(dats = dats, level_vec = level_vec, biomass_type = "dry")
#'
#' # The estimates, joined back to the data
#' res$data[, c("genus", "size_col", "biomass_col", "size_level", "model_level")]
#'
#' # The size estimates and the models they came from
#' res$size_estimates
#' res$model_estimates
#' @export
hellometry <- function(dats,
                       level_vec,
                       biomass_type = "dry",
                       model = "lm",
                       r_square_cutoff_upper = 0.95,
                       r_square_cutoff_lower = 0,
                       p_val_cutoff = 0.05) {

  # Columns must be properly named, biomass_type and model must have valid values
  data_checker(dats = dats,
               biomass_type = biomass_type,
               model = model)

  # Make copy of dataset to then reuse
  ret <-
    dats %>%
    ## Add row numbers, to join back to original data at the end
    dplyr::mutate(row = row_number())

  # Make measurement table
  ## Print message
  message("Making measurement table...")
  ## Make table
  measurement_table <-
    make_measurement_table(dats = dats,
                           level_vec = level_vec)

  # Getting size estimates
  ## Print message
  message("Getting size estimates...")
  ## Make table with all estimates
  full_estimation_table_size <- 
    full_estimation_table(level_vec = level_vec, 
                          measurement_table = measurement_table,
                          what = "size_col",
                          r_square_cutoff_upper = r_square_cutoff_upper,
                          r_square_cutoff_lower = r_square_cutoff_lower,
                          p_val_cutoff = p_val_cutoff) 
  
  # Join size estimates to the original data
  size_estimates <- 
    ret %>% 
    ## Filter numerical measurements
    dplyr::filter(size_col %in% c("small", "medium", "large", "unknown")) %>% 
    ## Select columns we need
    ## Only keep columns we want
    dplyr::select(row, rev(dplyr::any_of(level_vec)), 
                  stage, size_col) %>% 
    ## Rename size category to join to estimation result
    dplyr::rename(size_category = size_col) %>% 
    ## Loop around taxonomic levels and add to main data
    purrr::reduce(
      ### Levels to loop around
      level_vec[level_vec != "traits"],
      ### Data to be used as base (non_numeric_taxa)
      .init = .,
      ### Join data based on series of filter
      .f = ~ .x %>%
        dplyr::left_join(full_estimation_table_size %>%
                           ### Filter rows based on current trophic level
                           dplyr::filter(level == .y) %>%
                           ### Rename taxon_name to match the corresponding level column (e.g., "genus")
                           ### Rename size to a level-specific name (e.g., "genus_size") to avoid overwriting
                           dplyr::select(!!sym(.y) := name, ### !!sym() because we are pasting character string that needs to be interpreted as column name 
                                         !!paste0(.y, "_size_col") := size_col, 
                                         stage, size_category),
                         ### Join by column name, stage and size category to get a new column for that trophic level
                         by = c(.y, "stage", "size_category"))) %>%
    ## Now that all estimates at all levels have been joined, pick the one at the smallest taxonomic level
    dplyr::mutate(
      ### Coalesce all size_col columns to get the first non-NA value
      ### Follows the order of the given vector!
      size_col = dplyr::coalesce(!!!syms(paste0(level_vec[level_vec != "traits"], 
                                               "_size_col"))),
      ### Make a new column that indicates the taxonomic level at which the estimation was kept
      ### case_when chooses the first TRUE condition and assigns its corresponding value (the taxonomic level name)
      size_level = dplyr::case_when(!!!purrr::imap(paste0(level_vec[level_vec != "traits"], 
                                                          "_size_col"), 
                                                   ### Check which one of the values comes first as non-NA
                                                   ~ expr(!is.na(!!sym(.x)) ~ ### !!sym() because we are pasting character string that needs to be interpreted as column name
                                                            !!level_vec[level_vec != "traits"][.y]))),
      ### Finally, create a new column that contains the taxon name at the level of estimation
      size_taxon_name = dplyr::case_when(
        !!!imap(level_vec[level_vec != "traits"], ~ 
                  expr(size_level == !!.x ~ !!sym(.x))))) %>%
    ## Only keep columns we want
    dplyr::select(row, stage,
                  size_col, size_category,
                  size_level, size_taxon_name) 
  
  # Getting biomass models
  ## Print message
  message("Getting biomass models...")
  ## Make table
  ### Some models will be bad but filtered out,
  ### Wrap in suppressWarnings to avoid cluttering
  suppressWarnings(
    full_estimation_table_biomass <- 
      full_estimation_table(level_vec = level_vec,
                            measurement_table = dry_wet(measurement_table,
                                                        biomass_type = biomass_type),
                            what = "biomass_col",
                            model = model,
                            r_square_cutoff_upper = r_square_cutoff_upper,
                            r_square_cutoff_lower = r_square_cutoff_lower,
                            p_val_cutoff = p_val_cutoff))
  
  # Join models to the original data
  model_estimates <- 
    ret %>% 
    ## Get NA biomasses
    dplyr::filter(is.na(biomass_col)) %>% 
    ## Select columns we need
    dplyr::select(row, dplyr::any_of(level_vec), 
                  stage, biomass_col, biomass_type) %>%
    ## Loop around taxonomic levels and add to main data
    purrr::reduce(
      ### Levels to loop around
      level_vec[level_vec != "traits"],
      ### Data to be used as base (non_numeric_taxa)
      .init = .,
      ### Join data based on series of filter
      .f = ~ .x %>%
        dplyr::left_join(full_estimation_table_biomass %>%
                           ### Filter rows based on current trophic level
                           dplyr::filter(level == .y) %>%
                           ### Rename taxon_name to match the corresponding level column (e.g., "genus")
                           ### Rename size to a level-specific name (e.g., "genus_size") to avoid overwriting
                           dplyr::select(!!sym(.y) := name, ### !!sym() because we are pasting character string that needs to be interpreted as column name 
                                         stage, !!paste0(.y, "_model") := model),
                         ### Join by column name, stage and size category to get a new column for that trophic level
                         by = c(.y, "stage"))) %>%
    ## Now that all models at all levels have been joined, pick the one at the smallest taxonomic level
    ## Things are a bit trickier because the column consists of a list of models, not numerical values
    dplyr::mutate(
      ## Add a new column that contains the first non-NA model
      model = purrr::pmap(dplyr::select(., 
                                        dplyr::all_of(paste0(level_vec[level_vec != "traits"], 
                                                             "_model"))), 
                          function(...) {
                            ### Convert input to list
                            models <- 
                              list(...)
                            ### Find the first non-null model
                            first_non_null <- 
                              purrr::keep(models, 
                                          ~ !is.null(.x))
                            ### Return the first non-null model or NA if none found
                            if (length(first_non_null) > 0) 
                              first_non_null[[1]] else NA}),
      ## Get the corresponding level for that model
      model_level = purrr::pmap_chr(dplyr::select(., 
                                                  dplyr::all_of(paste0(level_vec[level_vec != "traits"], 
                                                                       "_model"))), 
                                    function(...) {
                                      ### Convert input to list
                                      models <- 
                                        list(...)
                                      ### Get the index of the first non-NULL model
                                      level_idx <- 
                                        which(purrr::map_lgl(models, 
                                                             ~ !is.null(.x)))
                                      ### Return the corresponding level or NA if none found
                                      if (length(level_idx) > 0) 
                                        level_vec[level_vec != "traits"][[level_idx[1]]] else NA_character_}),
      ## Get the corresponding taxon name from which the model came
      model_taxon_name = dplyr::case_when(
        !!!imap(level_vec[level_vec != "traits"], 
                ~ expr(model_level == !!.x ~ !!sym(.x))))) %>% 
    ## Only keep columns we want
    dplyr::select(row, stage,
                  model,  model_level, 
                  model_taxon_name) 
  
  # Stick back to original data
  ## Print message
  message("Combining estimates and models with data...")
  ## Stick back to the data
  ### Suppress warnings when mutated to numeric
  suppressWarnings(
    ret <- 
      ret %>% 
      ## Join size estimates by row number 
      dplyr::left_join(size_estimates %>% 
                         dplyr::select(-stage),
                       by = "row") %>% 
      ## Combine by new and old sizes
      dplyr::mutate(size_col.x = as.numeric(size_col.x),
                    size_col = dplyr::coalesce(size_col.x, size_col.y)) %>%
      ## Join models by row number
      dplyr::select(-size_col.x, -size_col.y) %>% 
      dplyr::left_join(model_estimates %>% 
                         dplyr::select(-stage),
                       by = "row") %>%
      ## Now get estimates of biomass for the whole row
      dplyr::mutate(
        ### Iterate over the list column
        biomass = purrr::pmap(
          list(model, size_col, abundance),
          function(model, size_col, abundance) {
            ### If the cell is NULL (biomass was inputted),make NA
            if (is.null(model)) {
              NA}
            ### If the cell is NA (no model could be computed), make NA
            else if (length(model) == 0 || any(is.na(model))) {
              NA}
            else {
              ### if we have all the data we need, use predict to get estimate
              ### predict_biomass() scales to abundance and back-transforms
              predict_biomass(model = model,
                              size = size_col,
                              abundance = abundance)}})) %>%
      ## Do rowwise operations
      dplyr::rowwise() %>%
      ## Split the estimate and its bounds into their own columns
      dplyr::mutate(biomass_lower = biomass[2],
                    biomass_upper = biomass[3],
                    biomass = biomass[1]) %>%
      ## Ungroup
      dplyr::ungroup() %>%
      ## Coalesce the two biomass columns
      dplyr::mutate(biomass_col = dplyr::coalesce(biomass, biomass_col)) %>%
      ## Remove biomass column
      dplyr::select(-biomass) %>%
      ## If we have a value in size_col but no size estimate, make it "raw_data"
      dplyr::mutate(dplyr::across(c(size_category, size_level, size_taxon_name),
                                  ~ ifelse(is.na(.),
                                           "raw_data", .))) %>% 
      ## If we have no biomass estimate say that estimation failed
      dplyr::mutate(dplyr::across(c(model_level, model_taxon_name),
                                  ~ ifelse(is.na(biomass_col),
                                           "estimation_failed", .))) %>% 
      ## Finally if we have NAs left in model_level and model_taxon_name, make them "raw_data"
      dplyr::mutate(dplyr::across(c(model_level, model_taxon_name),
                                  ~ ifelse(is.na(.),
                                           "raw_data", .))) %>% 
      ## I am a bit of a maniac so have size_col before biomass_col, and intevals after
      dplyr::relocate(size_col, 
                      .before = "biomass_col") %>% 
    dplyr::relocate(biomass_lower, biomass_upper, 
                    .after = "biomass_col"))
  
  
  
  # Return a list of three dataframes
  ## The original data with estimates, the used size estimates, and the used models
  return(list(
    ## Original data with estimates
    data = 
      ret %>% 
      ### Remove row number
      dplyr::select(-row),
    ## Size estimation
    size_estimates = 
      size_estimates %>% 
      ### Remove NAs
      dplyr::filter(!is.na(size_col)) %>%
      ### Make longer format 
      dplyr::select(level = size_level, 
                    name = size_taxon_name, 
                    stage, size_category, size_col) %>% 
      ### Keep unique rows
      unique(),
    ## Model estimation
    model_estimates = 
      model_estimates %>% 
      ### Remove NAs
      dplyr::filter(!is.na(model)) %>%
      ### Make longer format 
      dplyr::select(level = model_level, 
                    name = model_taxon_name, 
                    stage, model) %>% 
      ### Keep unique rows
      unique()))
  
}

#' Read the `trini_communities` dataset
#'
#' Reads the `trini_communities` dataset, a set of invertebrate
#' community samples from 17 Trinidadian bromeliads (from @Rogy2024). It is provided
#' as a toy dataset to try out the package, alongside [bromeliad_inverts_measurements()].
#' Each taxon also carries a set of discrete (fuzzy) trait columns, matched from a
#' trait database (@Cereghino2018), that can be returned with `traits = TRUE` to
#' illustrate the trait-based grouping functions.
#'
#' @param traits Logical; if `TRUE`, the returned tibble includes the fuzzy
#' trait columns. If `FALSE` (the default), only the taxonomy, abundance,
#' bromeliad ID and stage columns are returned.
#' @return A tibble with one row per taxon per bromeliad, with columns for
#' taxonomy, abundance, and bromeliad ID, and, when `traits = TRUE`, a set of
#' discrete (fuzzy) trait columns.
#' @examples
#' # The communities the package ships with
#' trini_communities()
#'
#' # The same, with the fuzzy trait columns
#' trini_communities(traits = TRUE)
#' @export
trini_communities <- function(traits = FALSE) {

  # Locate the dataset shipped with the package
  path <-
    system.file("extdata", "trini_communities.csv",
                package = "hellometry",
                mustWork = TRUE)

  # Read it as a tibble
  dats <-
    tibble::as_tibble(
      utils::read.csv(path,
                      check.names = FALSE,
                      fileEncoding = "UTF-8",
                      stringsAsFactors = FALSE))

  # Drop the fuzzy trait columns unless the user asks for them
  if (!traits) {
    community_cols <-
      c("bromeliad_id", "class", "order", "family", "subfamily",
        "genus", "species", "n", "stage")
    dats <-
      dats[, intersect(community_cols, names(dats)), drop = FALSE]
  }

  dats

}

#' Returns size and biomass estimates for every taxon in measurement table
#'
#'
#' @param level_vec Vector of taxonomic levels over which to iterate estimation
#' @param measurement_table A table with the numerical measurements and biomass 
#' used to compute allometric lms 
#' @param what Should size (what = "size_col") or biomass (what = "biomass_col") be 
#' estimated? 
#' @param model What kind of model should be computed, "lm" (default) for a linear model on the log10-log10 scale, or "poisson" for a Poisson glm with a log link
#' @param traits Should the table be computed by traits or not? Default FALSE
#' @param trait_columns List of traits to match, should be column names in measurement_table
#' @param id_col Name of the column holding a unique identifier per species/taxon,
#' used only when `traits = TRUE`. Default "species".
#' @param r_square_cutoff_upper Upper cutoff for R2 in allometric models, models with values above it will not be used in estimation. Default is 0.95 to avoid overfit models
#' @param r_square_cutoff_lower Lower cutoff for R2 in allometric models, models with values below it will not be used in estimation. Default is 0.
#' @param p_val_cutoff Upper cutoff for p-value of allometric models, models with p_value above it will not be used in estimation. Default is 0.05.
#' @return Tibble with size estimates or allometric models used in estimation. 
#' @examples
#' # Reference measurements for one family, renamed to the columns the package expects
#' measurements <-
#'   bromeliad_inverts_measurements() %>%
#'   dplyr::filter(family == "Culicidae") %>%
#'   dplyr::rename(size_col = body_size_mm,
#'                 biomass_col = body_mass_mg,
#'                 biomass_type = mass_type) %>%
#'   dplyr::mutate(size_col = as.character(size_col),
#'                 biomass_col = as.numeric(biomass_col))
#'
#' # Taxa to estimate, with no measurement of their own
#' communities <-
#'   trini_communities() %>%
#'   dplyr::filter(family == "Culicidae") %>%
#'   dplyr::rename(abundance = n) %>%
#'   dplyr::mutate(size_col = "unknown",
#'                 biomass_col = NA,
#'                 biomass_type = "dry")
#'
#' # The two together are what the package works on
#' dats <-
#'   dplyr::bind_rows(communities, measurements)
#' level_vec <-
#'   c("species", "genus", "family")
#'
#' measurement_table <-
#'   make_measurement_table(dats = dats, level_vec = level_vec)
#'
#' # Every size estimate the data allows
#' full_estimation_table(level_vec = level_vec,
#'                       measurement_table = measurement_table,
#'                       what = "size_col")
#'
#' # Every allometric model the data allows, on dry biomass
#' full_estimation_table(level_vec = level_vec,
#'                       measurement_table = dry_wet(measurement_table,
#'                                                   biomass_type = "dry"),
#'                       what = "biomass_col")
#' @export
#' 
full_estimation_table <- function(level_vec, measurement_table,
                                  what, model = "lm", traits = FALSE,
                                  trait_columns = c(), id_col = "species",
                                  r_square_cutoff_upper = 0.95,
                                  r_square_cutoff_lower = 0,
                                  p_val_cutoff = 0.05){

  # If we want to get estimates with traits
  if (traits) {
    ## Use trait functions to make groupings of similar trait species
    ret <-
      make_trait_table(measurement_table = measurement_table,
                       trait_columns = trait_columns,
                       id_col = id_col)

  } else {
  # If we do not want traits
  # Get all possible combinations of data
    ret <-
      ## Split into list of tibbles, which each containing level and size_col
      purrr::map(
        level_vec[level_vec != "traits"],
        ~ measurement_table %>%
          ## Select only the level and size_col
          dplyr::select(name = dplyr::all_of(.x), 
                        stage, size_col, biomass_col) %>%
          ## Drop rows with a missing taxon name (NA or "") so the estimation
          ## falls back to the level above, and rows without a numeric size
          dplyr::filter(!is.na(name) & name != "" & !is.na(size_col)) %>%
          ## Add column with level name
          dplyr::mutate(level = .x)) %>% 
      ## Remove empty levels 
      purrr::keep(function(x) nrow(x) > 0) %>% 
      ## Filter out those groups with less than three unique measurements
      purrr::map(
        .,
        ~ .x %>%
            ## Group by level, name and stage
            dplyr::group_by(level, name, stage) %>%
            ## Filter out those with less than three unique size_col
            dplyr::filter(dplyr::n_distinct(size_col) >= 3) %>%
            ## Ungroup to avoid future issues
            dplyr::ungroup()) %>% 
      ## Remove empty levels 
      purrr::keep(function(x) nrow(x) > 0)}
    

  # If size we need the weighted average and size bins
    if(what == "size_col"){
      ret <- 
        ret %>% 
        purrr::map(.,
                   ~ .x %>%
                     ## Group by name
                     dplyr::group_by(level, name, stage) %>%
                     ## Make a column with terciles of sizes
                     dplyr::mutate(size_group = ntile(size_col, 3)) %>%
                     ## Group by size group and get mean of each
                     dplyr::group_by(level, name, stage, size_group) %>%
                     dplyr::summarise(
                       size_col = mean(size_col),
                       .groups = "drop") %>%
                     ## Create a new column with size category
                     dplyr::mutate(size_category = dplyr::case_when(
                       size_group == 1 ~ "small",
                       size_group == 2 ~ "medium",
                       size_group == 3 ~ "large")) %>%
                     ## Keep relevant columns
                     dplyr::select(level, name, stage, size_category, size_col) %>% 
                     ## Add overall mean
                     dplyr::bind_rows(
                       .x %>%
                         # Get average grouping by level and name
                         group_by(level, name, stage) %>%
                         summarise(size_col = mean(size_col), 
                                   .groups = "drop") %>%
                         ### Make size category column (to fit this average with unknown)
                         dplyr::mutate(size_category = "unknown"))) %>% 
        ## Flatten
        purrr::list_c()}
                   
                   
    # If biomass we need allometric equations               
    if(what == "biomass_col") {
      ## Prepare data in common for models
      ret <- 
        ret %>% 
        ### Since some organisms can have known size but unknown biomass
        ### We also need to filter here!
        purrr::map(.,
                   ~ .x %>%
                     ### First remove all NAs in biomass
                     dplyr::filter(!is.na(biomass_col)) %>% 
                     ### Group by level, name and stage
                     dplyr::group_by(level, name, stage) %>%
                     ### Filter out those with less than three unique size_col
                     dplyr::filter(dplyr::n_distinct(biomass_col) >= 3)) %>% 
        ## Remove empty levels 
        purrr::keep(function(x) nrow(x) > 0)
      
      ## If we want basic lms
      ## The log10-log10 scale is what makes the relationship allometric
      if(model == "lm"){
        ret <-
          ret %>%
          purrr::map(.,
                     ### Make a model column
                     ~ .x %>%
                       dplyr::summarise(
                         model = list(lm(log10(biomass_col) ~ log10(size_col),
                                    data = dplyr::pick(size_col, biomass_col))),
                         .groups = "drop"))}
      ## If we want Poisson glms
      ## Biomass is not log10-transformed here, the log link already models it
      ## on the log scale, so only size needs transforming
      if(model == "poisson"){
        ret <-
          ret %>%
          purrr::map(.,
                     ### Make a model column
                     ~ .x %>%
                       dplyr::summarise(
                         model = list(glm(biomass_col ~ log10(size_col),
                                    data = dplyr::pick(size_col, biomass_col),
                                    family = poisson)),
                         .groups = "drop"))}

      ## Filter out the bad models, the same way whatever their kind
      ret <-
        ret %>%
        purrr::map(.,
                   ~ .x %>%
                     ### Get the R2 of each model, and the p-value of its slope
                     dplyr::mutate(
                       .r2   = purrr::map_dbl(model, r_squared),
                       .pval = purrr::map_dbl(model,
                                              ~ summary(.x)$coefficients[2, 4])) %>%
                     ### Filter out models based on R2 cutoffs and p value
                     dplyr::filter(.r2 < r_square_cutoff_upper,
                                   .r2 > r_square_cutoff_lower,
                                   .pval < p_val_cutoff) %>%
                     ### Drop helper columns
                     dplyr::select(-.r2, -.pval))


      ## Convert data to format we want
      ret <- 
        ret %>% 
        ## Flatten
        purrr::list_c() %>% 
        ## Ungroup
        dplyr::ungroup()
    }
      
  # Return
  return(ret)
}

#' Parsing dry and wet measurements
#'
#' Filter dataframe based on required biomass type (dry or wet).
#'
#' @param dats Input data
#' @param biomass_type If biomass should be wet or dry, or most common
#' @return Dataframe with filtered biomass type, and chosen one (dry or wet)
#' @examples
#' # Data holding both kinds of biomass
#' dats <-
#'   data.frame(species = c("sp_a", "sp_b"),
#'              stage = "larva",
#'              size_col = c(3.1, 2.2),
#'              biomass_col = c(0.4, 0.2),
#'              biomass_type = c("dry", "wet"))
#'
#' # Keep the dry measurements only
#' dry_wet(dats, biomass_type = "dry")
#' @export
dry_wet <- function(dats, biomass_type){

  # Simply filter based on chosen category
  dats <-
    dats %>%
    ## Filter based on input
    ## .data$ is the column, .env$ is the function argument
    ## (without these, both sides resolve to the column and nothing is filtered)
    dplyr::filter(.data$biomass_type == .env$biomass_type)
  
  # Return data
  return(dats)
}

#' Not In Operator
#'
#' Returns logical vector indicating if elements are **not** in a given set.
#'
#' @param x vector or NULL: the values to be matched.
#' @param table vector or NULL: the values to be matched against.
#' @return A logical vector of the same length as `x`.
#' @examples
#' c("a", "b") %notin% c("b", "c")
#' @export
`%notin%` <- function(x, table) {
  !(x %in% table)
}

#' Read the `bromeliad_inverts_measurements` dataset
#'
#' Reads the  `bromeliad_inverts_measurements` dataset, a compilation of real invertebrate
#' body size (mm) and body mass (mg) measurements from bromeliad communities. If you use the
#' data outside the package, please cite the package.
#'
#' @return A tibble with real body length and body size measurements from bromeliad invertebrates.
#' It contains columns for taxonomy ("phylum" to "species"), "stage", "abundance", "body_size_mm",
#' "body_mass_mg", "mass_type" (dry/wet), "data_providers" and "note".
#' @examples
#' # The measurements the package ships with
#' bromeliad_inverts_measurements()
#' @export
bromeliad_inverts_measurements <- function() {

  # Locate the dataset shipped with the package
  path <-
    system.file("extdata", "bromeliad_inverts_measurements.csv",
                package = "hellometry",
                mustWork = TRUE)

  # Read it and return as a tibble
  tibble::as_tibble(
    utils::read.csv(path,
                    check.names = FALSE,
                    fileEncoding = "UTF-8",
                    stringsAsFactors = FALSE))

}

#' Prepare data to do estimation with traits in `full_estimation_table`
#'
#' Remove columns with taxonomy, and replace them with trait-based groupings
#'
#' @param measurement_table A table with the numerical measurements and biomass
#' used to compute allometric lms
#' @param trait_columns List of traits to match, should be column names in measurement_table
#' @param id_col Name of the column holding a unique identifier per species/taxon.
#' Default "species".


#' @return List of tibbles, one per trait grouping, each carrying the
#' measurements of the species sharing that grouping's traits
#' @examples
#' # Fuzzy traits of three species, sp_a and sp_b within +/- 1 of each other
#' measurement_table <-
#'   data.frame(species = rep(c("sp_a", "sp_b", "sp_c"), each = 4),
#'              stage = "larva",
#'              size_col = c(1, 2, 3, 4, 1.2, 2.1, 3.3, 4.2, 8, 9, 10, 11),
#'              biomass_col = c(0.1, 0.4, 1, 2, 0.2, 0.5, 1.1, 2.2, 12, 15, 20, 26),
#'              AS1 = rep(c(1, 2, 5), each = 4),
#'              AS2 = rep(c(3, 3, 9), each = 4))
#'
#' # The measurements of the species sharing each trait grouping
#' make_trait_table(measurement_table, trait_columns = c("AS1", "AS2"))
#' @export
#'
#'
make_trait_table <- function(measurement_table,
                             trait_columns, id_col = "species") {

  # Get long format data of matching traits
  matched_traits <-
    matcher_of_traits(measurement_table = measurement_table,
                      trait_columns = trait_columns,
                      id_col = id_col)


  # Print message to say that we are cleaning the data now
  message("Cleaning trait groupings..")

  # Combine the two together
  ret <-
    matched_traits %>%
    ## Remove species without match
    dplyr::group_by(level, name, stage) %>%
    dplyr::filter(n() > 1) %>%
    dplyr::ungroup() %>%
    ## Split into set of lists
    dplyr::group_split(name,
                       .keep = T) %>%
    ## Add measurements for all species
    purrr::map(.,
               ~ .x %>%
                 ## Add measurements, joining matched ids to the id column
                 dplyr::left_join(measurement_table %>%
                                    ## Keep important columns only
                                    dplyr::select(dplyr::all_of(id_col),
                                                  size_col, biomass_col),
                                  by = stats::setNames(id_col, "id"),
                                  ## Just because one species can have several measurements
                                  relationship = "many-to-many") %>%
                 ## Remove all NAs in sizes
                 dplyr::filter(!is.na(size_col))) %>%
    ## Filter out those groups with less than three unique measurements
    purrr::map(.,
               ~ .x %>%
                 ## Group by level, name and stage
                 dplyr::group_by(level, name, stage) %>%
                 ## Filter out those with less than three unique size_col
                 dplyr::filter(dplyr::n_distinct(size_col) >= 3) %>%
                 ## Ungroup to avoid future issues
                 dplyr::ungroup()) %>%
    ## Remove empty levels
    purrr::keep(function(x) nrow(x) > 0)

  # Return
  return(ret)

}

---
title: "hellometry"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{hellometry}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message = FALSE)
```
## Introduction to package use
Welcome to hellometry! This simple, lightweight package allows you to quickly
impute missing body size and body mass measurements in the supplied data frame (or tibble). The package uses existing measurements in your data to estimate the missing ones, so, essentially, the better the data supplied the better the estimation will be. It is compatible with any kind of data, provided you follow a few simple rules of column naming. These are as follow: <br>
- column with measurements needs to be called "size_col", and only accepts numerical measurements or "unknown", "small", "medium" or "large" as categorical values <br>
- column with development stage of specimens (larva/adult for instance) needs to be called "stage" <br>
- column with abundance of specimens needs to be called "abundance" <br>
- column with biomass of specimens needs to be called 'biomass_col'
- column with biomass type needs to be called 'biomass_type", and only accepts "dry" or "wet"

The idea of the package, wrapped in `hellometry()` is rather simple. First, a `measurement_table` is compiled from your own measurements. This table is then used to compute every possible size estimates, and allometric models with  `full_estimation_table()`. The `model` argument picks the kind of allometric model: `"lm"` (the default) fits a linear model on the log10-log10 scale, while `"poisson"` fits a Poisson glm with a log link. Models are kept only if $0 < R^2 < 0.95$ and $p$-value $< 0.05$, and discarded otherwise, unless you specify other values. The $R^2$ comes from `performance::r2()`. Note that size and models are not necessarily computed at the same scale, it all depends on model fit filtered by these $p$ and $R^2$ values! Finally, the size estimate and model at the lowest possible taxonomic level are joined back to the data, and used to estimate biomass at that particular row. The estimates are given with a confidence interval, i.e. the uncertainty around the mean predicted biomass, scaled to the abundance of the row.
`hellometry()` returns a list of three elements: 
- your data with estimated body sizes and biomasses, and column with information on the level at which were performed the estimation, 
- a tibble with all unique size estimates that were joined to your data, 
- a tibble with all unique allometric models that were joined to your data. <br>

The flowchart below traces one `hellometry()` call, from your data to the returned list.

```{r flowchart, echo=FALSE, out.width="100%", fig.alt="Flowchart of a hellometry() call, from the input data through data_checker(), make_measurement_table(), full_estimation_table() for sizes then models, and predict_biomass(), to the returned list of three tables."}
knitr::include_graphics("hellometry_flowchart.svg")
```



## Quick working example
The package estimates sizes and biomasses from the measurements you provide.
To illustrate, the package ships with two real datasets from Trinidadian bromeliad communities [(Rogy et al., 2024)](https://doi.org/10.1111/ele.14391): `bromeliad_inverts_measurements()`, a large set of invertebrate body size (head to tail, mm) and body mass (mg) measurements compiled by the authors, and `trini_communities()`, the taxa observed in a set of bromeliads for which we want size and biomass estimates. <br>
```{r}
# Load library
library(hellometry)

# We need to define a suite of levels to be used for estimation, i.e. columns in the data, 
# from lowest to coarsest resolution
level_vec  <-
  c("species", "genus", "family", "order")

# Read in the reference measurements, the real body sizes and body masses used to build the models
measurements <-
  bromeliad_inverts_measurements() %>%
  ## Rename columns to those expected by hellometry()
  dplyr::rename(size_col = body_size_mm,
                biomass_col = body_mass_mg,
                biomass_type = mass_type) %>%
  ## size_col holds both numbers and categories, so it must be character;
  ## biomass_col must be numeric for the allometric models
  dplyr::mutate(size_col = as.character(size_col),
                biomass_col = as.numeric(biomass_col))

# Have a look at the reference measurements
dplyr::glimpse(measurements)

# Read in the Trinidadian communities, the taxa we want size and biomass estimates for
communities <-
  trini_communities() %>%
  ## Rename the abundance column and add the columns hellometry() needs
  dplyr::rename(abundance = n) %>%
  ## Here we do not have any measurement for these invertebrates, so size is "unknown"
  ## and biomass NA, to be estimated. We use dry biomass because it tends to be more precise
  dplyr::mutate(size_col = "unknown",
                biomass_col = NA,
                biomass_type = "dry")

# Have a look at the communities to estimate
dplyr::glimpse(communities)

# Combine the reference measurements and the target communities into one table
my_invertebrates <-
  communities %>%
  dplyr::bind_rows(measurements)


# Now we can use the package to estimate sizes and biomasses!

# Get the estimates
my_invertebrates_estimated <-
  hellometry(dats = my_invertebrates, ## The data to be used
              level_vec  = level_vec , ## The taxonomic levels to be used
              biomass_type = "dry") ## Type of biomass to be estimated

# The result is a list of three elements
# - `data` - your data with estimated body sizes and biomasses, and column with 
# information on the level at which were performed the estimation. Note that 
# it also includes the measurements supplied so will need to filter out these rows
dplyr::glimpse(my_invertebrates_estimated$data)
# - `size_estimates` - a tibble with all unique size estimates that were joined to your data
dplyr::glimpse(my_invertebrates_estimated$size_estimates)
# - `model_estimates` - a tibble with all unique allometric models that were joined to your data
dplyr::glimpse(my_invertebrates_estimated$model_estimates)


# But now let's say you do not really want to do estimation on the data, 
# but just see the results of all possible estimations. Fear not, 
# it is easy!
## First compile the measurement table
measurement_table <-
  make_measurement_table(dats = my_invertebrates, ## Data to be used
                         level_vec  = level_vec) ## Taxonomic levels to be used
## Get all possible size estimates
size_estimation_table <-
  full_estimation_table(level_vec  = level_vec ,
                        measurement_table = measurement_table,
                        what = "size_col")
### Have a glimpse of it
dplyr::glimpse(size_estimation_table)

## Get all possible models
model_estimation_table <-
  full_estimation_table(level_vec  = level_vec ,
                        measurement_table = dry_wet(measurement_table,
                                                        biomass_type = "dry"), ## Filter for dry biomass
                        what = "biomass_col")
### Have a glimpse of it
dplyr::glimpse(model_estimation_table)

```

## Alternative to taxonomical grouping: fuzzy traits
Let's say that you do not have valid models that feels satisfying under a rather high taxonomic level (e.g. order). Here, one alternative would be to use traits instead of taxonomy to infer similarities between species. For example, two small, white, oblong, worm-like creatures have a high likelihood of having similar body masses if of the same body size.<br>
The package contains a function to help you do so. So far, they are not incorporated into the `hellometry()` main function, but can get you as far as `full_estimation_table()`. To use them, you would need to supply in `measurement_table` a set of columns that contain <b>[fuzzy]( https://doi.org/10.1111/j.1365-2427.1994.tb01742.x)</b> traits. The functions will only work with fuzzy traits with discrete integer values, and will group species with those that have trait values +/-1 those of the focus species. Here is one example, using the discrete (fuzzy) traits that ship with `trini_communities()`, derived from an [existing study](https://doi.org/10.1111/1365-2435.13141) and assigning them to the measured taxa by taxonomic similarity. The beginning of the code is a bit technical, but shows you an example of how to incorporate fuzzy traits to your data in order to use that aspect of the package.
```{r}

# Read in data with traits
trait_data <-
  trini_communities(traits = TRUE)

# The fuzzy traits are every column after the taxonomy, i.e. from the first trait
# column (AS1) up to the last column of the table
trait_columns <-
  trait_data %>%
  ## Get the block of trait columns ..
  dplyr::select(AS1:dplyr::last_col()) %>%
  ## Keep just their names
  names()

# Rebuild a measurement table, this time asking it to keep every taxonomic level
# we want to match on (make_measurement_table() only keeps the levels we give it)
measurement_table <-
  make_measurement_table(dats = my_invertebrates,
                         level_vec = c("species", "genus", "subfamily",
                                       "family", "order", "class"))

# Here we want to match traits from different morphospecies, look for matches 
# from genus up to class
match_levels <-
  c("genus", "subfamily", "family", "order", "class")

# Build a lookup holding one representative trait profile per taxon, at 
# every level combined into a single long table that remembers the level 
# each profile came from
trait_lookup <-
  match_levels %>%
  purrr::map_dfr(.,
                 ~ trait_data %>%
                     ## Drop the taxa that are unnamed at this level
                     dplyr::filter(!is.na(.data[[.x]]), .data[[.x]] != "") %>%
                     ## One row per taxon name, carrying its trait profile
                     dplyr::distinct(key = .data[[.x]],
                                     dplyr::across(dplyr::all_of(trait_columns))) %>%
                     ## Keep a single profile should a taxon appear more than once
                     dplyr::distinct(key, .keep_all = TRUE) %>%
                     ## Tag which taxonomic level this profile belongs to
                     dplyr::mutate(match_level = .x))

# Now hand each measured taxon the trait profile of its most similar relative, i.e.
# the profile found at the finest taxonomic level at which the taxon matches
trait_assignment <-
  measurement_table %>%
  ## Tag each measurement row so we can return its assigned traits later
  dplyr::mutate(.row = dplyr::row_number()) %>%
  ## Keep only that tag and the taxonomy columns we match on
  dplyr::select(.row, dplyr::all_of(match_levels)) %>%
  ## Stretch to one row per measured taxon per taxonomic level
  tidyr::pivot_longer(dplyr::all_of(match_levels),
                      names_to = "match_level", values_to = "key") %>%
  ## Drop the levels where the taxon has no name
  dplyr::filter(!is.na(key), key != "") %>%
  ## Attach every trait profile that matches, at whatever level
  dplyr::inner_join(trait_lookup, 
                    by = c("match_level", "key")) %>%
  ## Rank the levels from finest to coarsest, following match_levels
  dplyr::mutate(.priority = match(match_level, 
                                  match_levels)) %>%
  ## For each measurement row, keep only the finest matching level
  dplyr::group_by(.row) %>%
  dplyr::slice_min(.priority, 
                   n = 1, 
                   with_ties = FALSE) %>%
  dplyr::ungroup() %>%
  ## Return just the row tag and its assigned trait profile
  dplyr::select(.row, dplyr::all_of(trait_columns))

# Finally, join the assigned traits back onto the measurements, and keep only taxa
# that both received traits and are identified to family (our id_col below)
dats <-
  measurement_table %>%
  ## Recreate the same row tag to join on
  dplyr::mutate(.row = dplyr::row_number()) %>%
  ## Bring in each taxon's assigned trait profile
  dplyr::left_join(trait_assignment, by = ".row") %>%
  ## Drop the helper tag
  dplyr::select(-.row) %>%
  ## Keep taxa that got traits and have a family to identify them by
  dplyr::filter(!is.na(.data[[trait_columns[1]]]),
                !is.na(family), family != "")

## Get long format data of which taxa have similar traits
matched_traits <-
  matcher_of_traits(measurement_table = dats,
                    trait_columns = trait_columns,
                    id_col = "family") ## here taxa are identified by family
### See the output
head(matched_traits)

## Get clean list for full_estimation_table, includes matcher_of_traits()
## Here you can see which taxa were grouped together
clean_trait_list <-
  make_trait_table(measurement_table = dats,
                   trait_columns = trait_columns,
                   id_col = "family")
### See the output
clean_trait_list[[1]]

# The last two functions are called in full_estimation_table if you add the trait arguments
## Size estimation table
size_estimation_table_traits <-
  full_estimation_table(level_vec  = c(),
                        measurement_table = dats,
                        traits = TRUE, ## switch for traits
                        trait_columns = trait_columns, ## vector of fuzzy trait columns
                        id_col = "family", ## where to group
                        what = "size_col")
### See the output
head(size_estimation_table_traits)

## Model table
model_estimation_table_traits <-
  full_estimation_table(level_vec  = c(),
                        measurement_table = dry_wet(dats,
                                                    biomass_type = "dry"), ## Filter for dry biomass
                        traits = TRUE, ## traits = TRUE
                        trait_columns = trait_columns, ## vector of fuzzy trait columns
                        id_col = "family", ## column identifying taxa
                        what = "biomass_col")
### See the output
head(model_estimation_table_traits)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hellometry-package.R
\docType{package}
\name{hellometry-package}
\alias{hellometry-package}
\title{hellometry: Biomass Estimation from Allometric Relationships}
\description{
Estimates body size and biomass of organisms from allometric relationships. It works with any data that follow a few column-naming conventions, using the measurements you supply to impute the ones that are missing.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/pierrerogy/hellometry}
  \item Report bugs at \url{https://github.com/pierrerogy/hellometry/issues}
}

}
\author{
\strong{Maintainer}: Pierre Rogy \email{pierre.rogy@gmail.com} (\href{https://orcid.org/0000-0002-3002-0059}{ORCID})

Other contributors:
\itemize{
  \item Olivier Dézerald (\href{https://orcid.org/0000-0002-9987-9865}{ORCID}) [contributor]
  \item Gustavo Q. Romero (\href{https://orcid.org/0000-0003-3736-4759}{ORCID}) [contributor]
  \item Fabiola Ospina-Bautista (\href{https://orcid.org/0000-0003-2498-1459}{ORCID}) [contributor]
  \item Sarah Abdelazim (\href{https://orcid.org/0000-0002-6379-2733}{ORCID}) [contributor]
  \item Nicholas A. C. Marino (\href{https://orcid.org/0000-0002-5702-5466}{ORCID}) [contributor]
  \item Diane S. Srivastava (\href{https://orcid.org/0000-0003-4541-5595}{ORCID}) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/make_trait_table.R
\name{make_trait_table}
\alias{make_trait_table}
\title{Prepare data to do estimation with traits in \code{full_estimation_table}}
\usage{
make_trait_table(measurement_table, trait_columns, id_col = "species")
}
\arguments{
\item{measurement_table}{A table with the numerical measurements and biomass
used to compute allometric lms}

\item{trait_columns}{List of traits to match, should be column names in measurement_table}

\item{id_col}{Name of the column holding a unique identifier per species/taxon.
Default "species".}
}
\value{
List of tibbles, one per trait grouping, each carrying the
measurements of the species sharing that grouping's traits
}
\description{
Remove columns with taxonomy, and replace them with trait-based groupings
}
\examples{
# Fuzzy traits of three species, sp_a and sp_b within +/- 1 of each other
measurement_table <-
  data.frame(species = rep(c("sp_a", "sp_b", "sp_c"), each = 4),
             stage = "larva",
             size_col = c(1, 2, 3, 4, 1.2, 2.1, 3.3, 4.2, 8, 9, 10, 11),
             biomass_col = c(0.1, 0.4, 1, 2, 0.2, 0.5, 1.1, 2.2, 12, 15, 20, 26),
             AS1 = rep(c(1, 2, 5), each = 4),
             AS2 = rep(c(3, 3, 9), each = 4))

# The measurements of the species sharing each trait grouping
make_trait_table(measurement_table, trait_columns = c("AS1", "AS2"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trini_communities.R
\name{trini_communities}
\alias{trini_communities}
\title{Read the \code{trini_communities} dataset}
\usage{
trini_communities(traits = FALSE)
}
\arguments{
\item{traits}{Logical; if \code{TRUE}, the returned tibble includes the fuzzy
trait columns. If \code{FALSE} (the default), only the taxonomy, abundance,
bromeliad ID and stage columns are returned.}
}
\value{
A tibble with one row per taxon per bromeliad, with columns for
taxonomy, abundance, and bromeliad ID, and, when \code{traits = TRUE}, a set of
discrete (fuzzy) trait columns.
}
\description{
Reads the \code{trini_communities} dataset, a set of invertebrate
community samples from 17 Trinidadian bromeliads (from @Rogy2024). It is provided
as a toy dataset to try out the package, alongside \code{\link[=bromeliad_inverts_measurements]{bromeliad_inverts_measurements()}}.
Each taxon also carries a set of discrete (fuzzy) trait columns, matched from a
trait database (@Cereghino2018), that can be returned with \code{traits = TRUE} to
illustrate the trait-based grouping functions.
}
\examples{
# The communities the package ships with
trini_communities()

# The same, with the fuzzy trait columns
trini_communities(traits = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/full_estimation_table.R
\name{full_estimation_table}
\alias{full_estimation_table}
\title{Returns size and biomass estimates for every taxon in measurement table}
\usage{
full_estimation_table(
  level_vec,
  measurement_table,
  what,
  model = "lm",
  traits = FALSE,
  trait_columns = c(),
  id_col = "species",
  r_square_cutoff_upper = 0.95,
  r_square_cutoff_lower = 0,
  p_val_cutoff = 0.05
)
}
\arguments{
\item{level_vec}{Vector of taxonomic levels over which to iterate estimation}

\item{measurement_table}{A table with the numerical measurements and biomass
used to compute allometric lms}

\item{what}{Should size (what = "size_col") or biomass (what = "biomass_col") be
estimated?}

\item{model}{What kind of model should be computed, "lm" (default) for a linear model on the log10-log10 scale, or "poisson" for a Poisson glm with a log link}

\item{traits}{Should the table be computed by traits or not? Default FALSE}

\item{trait_columns}{List of traits to match, should be column names in measurement_table}

\item{id_col}{Name of the column holding a unique identifier per species/taxon,
used only when \code{traits = TRUE}. Default "species".}

\item{r_square_cutoff_upper}{Upper cutoff for R2 in allometric models, models with values above it will not be used in estimation. Default is 0.95 to avoid overfit models}

\item{r_square_cutoff_lower}{Lower cutoff for R2 in allometric models, models with values below it will not be used in estimation. Default is 0.}

\item{p_val_cutoff}{Upper cutoff for p-value of allometric models, models with p_value above it will not be used in estimation. Default is 0.05.}
}
\value{
Tibble with size estimates or allometric models used in estimation.
}
\description{
Returns size and biomass estimates for every taxon in measurement table
}
\examples{
# Reference measurements for one family, renamed to the columns the package expects
measurements <-
  bromeliad_inverts_measurements() \%>\%
  dplyr::filter(family == "Culicidae") \%>\%
  dplyr::rename(size_col = body_size_mm,
                biomass_col = body_mass_mg,
                biomass_type = mass_type) \%>\%
  dplyr::mutate(size_col = as.character(size_col),
                biomass_col = as.numeric(biomass_col))

# Taxa to estimate, with no measurement of their own
communities <-
  trini_communities() \%>\%
  dplyr::filter(family == "Culicidae") \%>\%
  dplyr::rename(abundance = n) \%>\%
  dplyr::mutate(size_col = "unknown",
                biomass_col = NA,
                biomass_type = "dry")

# The two together are what the package works on
dats <-
  dplyr::bind_rows(communities, measurements)
level_vec <-
  c("species", "genus", "family")

measurement_table <-
  make_measurement_table(dats = dats, level_vec = level_vec)

# Every size estimate the data allows
full_estimation_table(level_vec = level_vec,
                      measurement_table = measurement_table,
                      what = "size_col")

# Every allometric model the data allows, on dry biomass
full_estimation_table(level_vec = level_vec,
                      measurement_table = dry_wet(measurement_table,
                                                  biomass_type = "dry"),
                      what = "biomass_col")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matcher_of_traits.R
\name{matcher_of_traits}
\alias{matcher_of_traits}
\title{Finds which species have matching traits}
\usage{
matcher_of_traits(measurement_table, trait_columns, id_col = "species")
}
\arguments{
\item{measurement_table}{A table with the numerical measurements and biomass
used to compute allometric lms}

\item{trait_columns}{List of traits to match, should be column names in measurement_table}

\item{id_col}{Name of the column holding a unique identifier per species/taxon.
Default "species".}
}
\value{
Tibble of four columns: level = "traits", name = focus species,
stage of focus species, id = matched species
}
\description{
Evaluates all unique trait groupings and returns species that have the same
traits +/- 1
}
\examples{
# Fuzzy traits of three species, sp_a and sp_b within +/- 1 of each other
measurement_table <-
  data.frame(species = rep(c("sp_a", "sp_b", "sp_c"), each = 4),
             stage = "larva",
             size_col = c(1, 2, 3, 4, 1.2, 2.1, 3.3, 4.2, 8, 9, 10, 11),
             biomass_col = c(0.1, 0.4, 1, 2, 0.2, 0.5, 1.1, 2.2, 12, 15, 20, 26),
             AS1 = rep(c(1, 2, 5), each = 4),
             AS2 = rep(c(3, 3, 9), each = 4))

# sp_a and sp_b match each other, sp_c only itself
matcher_of_traits(measurement_table, trait_columns = c("AS1", "AS2"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/make_measurement_table.R
\name{make_measurement_table}
\alias{make_measurement_table}
\title{Build measurement table to be used for biomass estimations}
\usage{
make_measurement_table(dats, level_vec)
}
\arguments{
\item{dats}{Dataframe to be used for estimation.}

\item{level_vec}{Vector of taxonomic levels to be used in the measurement table.}
}
\value{
A table with measurements and taxonomy, ready to be used for
size and biomass estimation.
}
\description{
Compiles the table of numerical measurements used to estimate sizes and fit
allometric models. The table is built entirely from the data you supply:
every row with a numerical \code{size_col} is kept (expanded by \code{abundance} so
each row represents one individual), carrying its taxonomy along.
}
\examples{
# Reference measurements for one family, renamed to the columns the package expects
measurements <-
  bromeliad_inverts_measurements() \%>\%
  dplyr::filter(family == "Culicidae") \%>\%
  dplyr::rename(size_col = body_size_mm,
                biomass_col = body_mass_mg,
                biomass_type = mass_type) \%>\%
  dplyr::mutate(size_col = as.character(size_col),
                biomass_col = as.numeric(biomass_col))

# Taxa to estimate, with no measurement of their own
communities <-
  trini_communities() \%>\%
  dplyr::filter(family == "Culicidae") \%>\%
  dplyr::rename(abundance = n) \%>\%
  dplyr::mutate(size_col = "unknown",
                biomass_col = NA,
                biomass_type = "dry")

# The two together are what the package works on
dats <-
  dplyr::bind_rows(communities, measurements)
level_vec <-
  c("species", "genus", "family")

# One row per measured individual, taxonomy carried along
make_measurement_table(dats = dats, level_vec = level_vec)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/r_squared.R
\name{r_squared}
\alias{r_squared}
\title{Get the R2 of a fitted allometric model}
\usage{
r_squared(model)
}
\arguments{
\item{model}{A fitted allometric model}
}
\value{
A single R2 value, NA if none could be computed
}
\description{
Wraps \code{performance::r2()}, which returns the R2 suited to the model it is
given, so that any kind of model can be filtered the same way in
\code{full_estimation_table()}
}
\examples{
# Any allometric model will do
mod <-
  lm(log10(biomass_col) ~ log10(size_col),
     data = data.frame(size_col = c(1, 2, 3, 4, 5),
                       biomass_col = c(0.1, 0.5, 1.2, 2.4, 4.1)))

r_squared(mod)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operators.R
\name{\%notin\%}
\alias{\%notin\%}
\title{Not In Operator}
\usage{
x \%notin\% table
}
\arguments{
\item{x}{vector or NULL: the values to be matched.}

\item{table}{vector or NULL: the values to be matched against.}
}
\value{
A logical vector of the same length as \code{x}.
}
\description{
Returns logical vector indicating if elements are \strong{not} in a given set.
}
\examples{
c("a", "b") \%notin\% c("b", "c")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bromeliad_inverts_measurements.R
\name{bromeliad_inverts_measurements}
\alias{bromeliad_inverts_measurements}
\title{Read the \code{bromeliad_inverts_measurements} dataset}
\usage{
bromeliad_inverts_measurements()
}
\value{
A tibble with real body length and body size measurements from bromeliad invertebrates.
It contains columns for taxonomy ("phylum" to "species"), "stage", "abundance", "body_size_mm",
"body_mass_mg", "mass_type" (dry/wet), "data_providers" and "note".
}
\description{
Reads the  \code{bromeliad_inverts_measurements} dataset, a compilation of real invertebrate
body size (mm) and body mass (mg) measurements from bromeliad communities. If you use the
data outside the package, please cite the package.
}
\examples{
# The measurements the package ships with
bromeliad_inverts_measurements()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hellometry.R
\name{hellometry}
\alias{hellometry}
\title{Wrapper function to estimate size (mm) and biomass (mg) from an input dataset}
\usage{
hellometry(
  dats,
  level_vec,
  biomass_type = "dry",
  model = "lm",
  r_square_cutoff_upper = 0.95,
  r_square_cutoff_lower = 0,
  p_val_cutoff = 0.05
)
}
\arguments{
\item{dats}{The input data table, please include columns "abundance",
"size_col", "biomass_col", "stage" (larva/pupa/adult).}

\item{level_vec}{A character vector indicating the taxonomic levels present
in your data, in increasing order of coarseness.}

\item{biomass_type}{"dry"/"wet". Should data used in inference be "dry" for
just dry biomass (default), or "wet" for just wet biomass. See \code{dry_wet()}
for more information on how this works.}

\item{model}{What kind of allometric model should be computed, "lm" (default)
for a linear model on the log10-log10 scale, or "poisson" for a Poisson glm
with a log link.}

\item{r_square_cutoff_upper}{Upper cutoff for R2 in allometric models, models with values above it will not be used in estimation. Default is 0.95 to avoid overfit models}

\item{r_square_cutoff_lower}{Lower cutoff for R2 in allometric models, models with values below it will not be used in estimation. Default is 0.}

\item{p_val_cutoff}{Upper cutoff for p-value of allometric models, models with p_value above it will not be used in estimation. Default is 0.05.}
}
\value{
A list with three tibbles:
\itemize{
\item data: the input data with added size and biomass estimates, and new columns
with the taxonomic level and name of the taxon at which the estimate
was made.
\item size_estimates: tibble with size estimates that were used,
with columns for taxonomic level, name and size category
of the estimation.
\item model_estimates: tibble with biomass models that were used, with columns
for taxonomic level and name.
See \code{full_estimation_table()} to get all possible size estimates and models
for your data
}
}
\description{
Wrapper function, input your data and get size and measurement.
Please name column with number of specimen "abundance", column with measurement
"size_col", column with life stage (e.g. larva/adult) "stage", column
with biomass type (dry/wet) "biomass_type", and the column with biomass
"biomass_col".
If you do not have a numerical measurement for a given specimen, the function
can do size estimations if you input "small", "medium", "large" or "unknown".
In this case, the algorithm will use existing size measurements for the species,
and use the size distribution to estimate "small", "medium" and "large" inputs,
or a weighted average of all measurements if the input is "unknown". For biomass
measurements, just leave NA for those cells you want estimates in.
}
\details{
Please make sure that the level_vec argument has levels in increasing order
of coarseness, e.g. from species to order.
}
\examples{
# Reference measurements for one family, renamed to the columns the package expects
measurements <-
  bromeliad_inverts_measurements() \%>\%
  dplyr::filter(family == "Culicidae") \%>\%
  dplyr::rename(size_col = body_size_mm,
                biomass_col = body_mass_mg,
                biomass_type = mass_type) \%>\%
  dplyr::mutate(size_col = as.character(size_col),
                biomass_col = as.numeric(biomass_col))

# Taxa to estimate, with no measurement of their own
communities <-
  trini_communities() \%>\%
  dplyr::filter(family == "Culicidae") \%>\%
  dplyr::rename(abundance = n) \%>\%
  dplyr::mutate(size_col = "unknown",
                biomass_col = NA,
                biomass_type = "dry")

# The two together are what the package works on
dats <-
  dplyr::bind_rows(communities, measurements)
level_vec <-
  c("species", "genus", "family")

res <-
  hellometry(dats = dats, level_vec = level_vec, biomass_type = "dry")

# The estimates, joined back to the data
res$data[, c("genus", "size_col", "biomass_col", "size_level", "model_level")]

# The size estimates and the models they came from
res$size_estimates
res$model_estimates
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict_biomass.R
\name{predict_biomass}
\alias{predict_biomass}
\title{Estimate biomass from a fitted allometric model}
\usage{
predict_biomass(model, size, abundance)
}
\arguments{
\item{model}{A fitted allometric model}

\item{size}{The size to estimate biomass for}

\item{abundance}{The number of individuals the row holds}
}
\value{
Three values: the biomass estimate, its lower and its upper bound
}
\description{
Wraps \code{predict()} for every kind of model \code{full_estimation_table()} can fit,
and returns the biomass of a whole row on the scale of the supplied data
}
\details{
Both kinds return a confidence interval, i.e. the uncertainty around the
mean predicted biomass. The estimate is scaled to the abundance of the row
while still on the scale the model was fitted on (log10 for an lm, the log
link for a Poisson glm), and then back-transformed.
}
\examples{
# Any allometric model will do
mod <-
  lm(log10(biomass_col) ~ log10(size_col),
     data = data.frame(size_col = c(1, 2, 3, 4, 5),
                       biomass_col = c(0.1, 0.5, 1.2, 2.4, 4.1)))

# Biomass of a row holding ten individuals of size 3, with its bounds
predict_biomass(mod, size = 3, abundance = 10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_checker.R
\name{data_checker}
\alias{data_checker}
\title{Set of checks to perform on the data to make sure it can be handled by \code{hellometry()}}
\usage{
data_checker(dats, biomass_type, model)
}
\arguments{
\item{dats}{Dataframe to be used for estimation}

\item{biomass_type}{"dry"/"wet". Whether the biomass used in inference should
be dry (default) or wet. See \code{dry_wet()} for details.}

\item{model}{"lm"/"poisson". What kind of allometric model should be computed.
See \code{full_estimation_table()} for details.}
}
\value{
An error if any check fails.
}
\description{
Will fail if any condition not met
}
\examples{
# Data named the way the package expects
dats <-
  data.frame(species = c("sp_a", "sp_b"),
             stage = "larva",
             abundance = c(2, 5),
             size_col = c("3.1", "unknown"),
             biomass_col = c(0.4, NA),
             biomass_type = "dry")

# Passes quietly
data_checker(dats, biomass_type = "dry", model = "lm")

# Fails on a column that is not named properly
try(data_checker(dats["species"], biomass_type = "dry", model = "lm"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dry_wet.R
\name{dry_wet}
\alias{dry_wet}
\title{Parsing dry and wet measurements}
\usage{
dry_wet(dats, biomass_type)
}
\arguments{
\item{dats}{Input data}

\item{biomass_type}{If biomass should be wet or dry, or most common}
}
\value{
Dataframe with filtered biomass type, and chosen one (dry or wet)
}
\description{
Filter dataframe based on required biomass type (dry or wet).
}
\examples{
# Data holding both kinds of biomass
dats <-
  data.frame(species = c("sp_a", "sp_b"),
             stage = "larva",
             size_col = c(3.1, 2.2),
             biomass_col = c(0.4, 0.2),
             biomass_type = c("dry", "wet"))

# Keep the dry measurements only
dry_wet(dats, biomass_type = "dry")
}
